# -*- coding: utf-8 -*-
"""Authenticates from the CLI"""

from oktaauthpy3 import metadata


__version__ = metadata.version
__author__ = metadata.authors[0]
__license__ = metadata.license
__copyright__ = metadata.copyright
