# -*- coding: utf-8 -*-
"""Project metadata

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'oktaauthpy3'
project = "Okta CLI authentication with Python3 support"
project_no_spaces = project.replace(' ', '')
version = '0.8'
description = 'Authenticates from the CLI'
authors = ['Peter Gillard-Moss']
authors_string = ', '.join(authors)
maintainers = ['Mansab Uppal']
maintainers_string = ', '.join(maintainers)
emails = ['muppal@thoughtworks.com']
license = 'Apache 2.0'
copyright = '2015 Thoughtworks Inc'
url = 'https://www.thoughtworks.com/'
